@@ -5,7 +5,7 @@ import os
 import sys
 from dataclasses import dataclass
 from io import StringIO
-from json import loads
+from json import decoder, loads
 from typing import Callable, List, Optional, Union
 
 import borg.archive
@@ -23,7 +23,7 @@ from .options import (
     ExclusionOptions,
     ExclusionOutput,
     FilesystemOptions,
-    _Options,
+    OptionsBase,
 )
 
 # pylint: disable=too-many-public-methods
@@ -40,38 +40,67 @@ class BorgAPI:
     ):
         self.options = options or {}
         self.defaults = defaults or {}
-        self.logger_setup = False
         self.archiver = borg.archiver.Archiver()
-        self.archiver.log_json = log_json
+        self._previous_dotenv = []
+        self._setup_logging(log_level, log_json)
+
+    def _setup_logging(self, log_level: str = "info", log_json: bool = False):
+        self.logger_setup = False
+        self.archiver.log_json = log_json or self.options.get("log_json", False)
         borg.archiver.setup_logging(level=log_level, is_serve=False, json=log_json)
         self.original_stdout = sys.stdout
-        self._previous_dotenv = []
+        logging.getLogger("borgapi")
+        self._logger = logging.getLogger(__name__)
 
     def _run(self, arg_list: List, func: Callable) -> Union[str, dict, None]:
-        capture = value = None
-        logging.debug("%s: %s", func.__name__, arg_list)
+        stdout_run = stderr_run = None
+        self._logger.debug("%s: %s", func.__name__, arg_list)
+        arg_list.insert(0, "borgapi")
         args = self.archiver.get_args(arg_list, os.environ.get("SSH_ORIGINAL_COMMAND"))
 
-        sys.stdout = temp_stdout = StringIO()
+        original_stderr = sys.stderr
+
+        sys.stdout = stdout_temp = StringIO()
+        sys.stderr = stderr_temp = StringIO()
         try:
             func(args)
         except Exception as e:
-            logging.error(e)
+            self._logger.error(e)
             raise e
         else:
-            value = temp_stdout.getvalue()
+            stdout_run = stdout_temp.getvalue().strip()
+            stderr_run = stderr_temp.getvalue().strip()
         finally:
+            sys.stdout.close()
             sys.stdout = self.original_stdout
-            temp_stdout.close()
+            sys.stderr = original_stderr
+            stdout_temp.close()
+            stderr_temp.close()
 
-        if getattr(args, "json", False) and value:
-            capture = loads(value)
-        elif value:
-            capture = value
+        if getattr(args, "json", False) or getattr(args, "json_lines", False):
+            stdout_json = stderr_json = None
+            if stdout_run:
+                try:
+                    stdout_json = loads(stdout_run)
+                except decoder.JSONDecodeError:
+                    clean_json = f'[{",".join(stdout_run.splitlines())}]'
+                    try:
+                        stdout_json = loads(clean_json)
+                    except decoder.JSONDecodeError:
+                        stdout_json = (stdout_run or None)
+            if stderr_run:
+                try:
+                    stderr_json = loads(stderr_run)
+                except decoder.JSONDecodeError:
+                    clean_json = f'[{",".join(stderr_run.splitlines())}]'
+                    try:
+                        stderr_json = loads(clean_json)
+                    except decoder.JSONDecodeError:
+                        stderr_json = (stderr_run or None)
+            return stdout_json, stderr_json
+        return (stdout_run or None), (stderr_run or None)
 
-        return capture
-
-    def _get_option_list(self, value: dict, options_class: _Options) -> List:
+    def _get_option_list(self, value: dict, options_class: OptionsBase) -> List:
         args = self.options | (value or {})
         return options_class(**args).parse()
 
@@ -79,7 +108,7 @@ class BorgAPI:
         self,
         command: str,
         values: dict,
-        options_class: _Options,
+        options_class: OptionsBase,
     ) -> List:
         optionals = self.defaults.get(command, {}) | (values or {})
         return options_class(**optionals).parse()
@@ -103,13 +132,13 @@ class BorgAPI:
         """
         variables = {}
         if filename:
-            logging.debug("Loading environment variables from %s", filename)
+            self._logger.debug("Loading environment variables from %s", filename)
             variables = dotenv_values(filename)
         elif dictionary or kwargs:
-            logging.debug("Loading dictionary with data: %s", variables)
+            self._logger.debug("Loading dictionary with data: %s", variables)
             variables = dictionary or kwargs
         else:
-            logging.debug('Looking for ".env" file to load variables from')
+            self._logger.debug('Looking for ".env" file to load variables from')
             variables = dotenv_values()
 
         self._previous_dotenv = variables.keys()
@@ -159,7 +188,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             append_only: bool = False
             storage_quota: str = None
             make_parent_dirs: bool = False
@@ -199,7 +228,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             dry_run: bool = False
             stats: bool = False
             list: bool = False
@@ -249,7 +278,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             list: bool = False
             dry_run: bool = False
             numeric_owner: bool = False
@@ -291,7 +320,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             repository_only: bool = False
             archives_only: bool = False
             verify_data: bool = False
@@ -361,7 +390,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             short: bool = False
             format: str = None
             json: bool = False
@@ -406,7 +435,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             numeric_owner: bool = False
             same_chunker_params: bool = False
             sort: bool = False
@@ -448,7 +477,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             dry_run: bool = False
             stats: bool = False
             cache_only: bool = False
@@ -489,7 +518,7 @@ class BorgAPI:
 
         # pylint: disable=too-many-instance-attributes
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             dry_run: bool = False
             force: bool = False
             stats: bool = False
@@ -536,7 +565,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             json: bool = False
 
             # pylint: disable=useless-super-delegation
@@ -577,7 +606,7 @@ class BorgAPI:
 
         # pylint: disable=invalid-name
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             foreground: bool = False
             o: str = None
 
@@ -664,7 +693,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             paper: bool = False
             qr_html: bool = False
 
@@ -702,7 +731,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             paper: bool = False
 
             # pylint: disable=useless-super-delegation
@@ -736,7 +765,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             dry_run: bool = False
             inplace: bool = False
             force: bool = False
@@ -779,7 +808,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             tar_filter: str = None
             list: bool = False
 
@@ -810,7 +839,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             restrict_to_path: str = None
             restrict_to_repository: str = None
             append_only: bool = False
@@ -848,7 +877,7 @@ class BorgAPI:
         """
 
         @dataclass
-        class _Optional(_Options):
+        class _Optional(OptionsBase):
             cache: bool = False
             delete: bool = False
             list: bool = False
@@ -875,7 +904,7 @@ class BorgAPI:
         repository: str,
         command: str,
         *args: Union[str, int],
-        **options: Union[bool, str, int]
+        **options: Union[bool, str, int],
     ) -> Union[str, dict, None]:
         """Run a user-specified command while the repository lock is held.
 
@@ -903,9 +932,7 @@ class BorgAPI:
         return self._run(arg_list, self.archiver.do_with_lock)
 
     def break_lock(
-        self,
-        repository: str,
-        **options: Union[bool, str, int]
+        self, repository: str, **options: Union[bool, str, int]
     ) -> Union[str, dict, None]:
         """Break the repository and cache locks.
 
@@ -927,10 +954,7 @@ class BorgAPI:
         return self._run(arg_list, self.archiver.do_break_lock)
 
     def benchmark_crud(
-        self,
-        repository: str,
-        path: str,
-        **options: Union[bool, str, int]
+        self, repository: str, path: str, **options: Union[bool, str, int]
     ) -> Union[str, dict, None]:
         """Benchmark borg CRUD (create, read, update, delete) operations.
 
